@@ -1,11 +1,11 @@
 package Lemonldap::NG::Handler::Lib::PSGI;
 
-use 5.10.0;
+#use 5.10.0;
 use Mouse;
 
 #use Lemonldap::NG::Handler::Main qw(:jailSharedVars);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.6';
 
 has protection => ( is => 'rw', isa => 'Str' );
 has rule       => ( is => 'rw', isa => 'Str' );
@@ -21,7 +21,7 @@ sub init {
         return 0;
     }
     unless ( $self->api->checkConf($self)
-        or $self->{protection} eq 'none' )
+        or ( $self->{protection} and $self->{protection} eq 'none' ) )
     {
         $self->error(
             "Unable to protect this server ($Lemonldap::NG::Common::Conf::msg)"
@@ -69,7 +69,7 @@ sub _run {
         return sub {
             my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
             my $res = $self->handler($req);
-            push @{ $res->[1] }, @{ $req->{respHeaders} };
+            push @{ $res->[1] }, $req->spliceHdrs;
             return $res;
         };
     }
@@ -79,16 +79,17 @@ sub status {
     my ( $class, $args ) = @_;
     $args //= {};
     my $self = $class->new($args);
+    $self->init($args);
 
     # Check if main handler initialization has been done
-    unless ( %$self->api->tsv ) {
+    unless ( %{ $self->api->tsv } ) {
         eval { $self->api->checkConf() };
         $self->logger->error($@) if ($@);
     }
     return sub {
         my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
         $self->api->status($req);
-        return [ 200, [ @{ $req->{respHeaders} } ], [ $req->{respBody} ] ];
+        return [ 200, [ $req->spliceHdrs ], [ $req->{respBody} ] ];
     };
 }
 
@@ -96,16 +97,17 @@ sub reload {
     my ( $class, $args ) = @_;
     $args //= {};
     my $self = $class->new($args);
+    $self->init($args);
 
     # Check if main handler initialization has been done
-    unless ( %$self->api->tsv ) {
+    unless ( %{ $self->api->tsv } ) {
         eval { $self->api->checkConf() };
         $self->logger->error($@) if ($@);
     }
     return sub {
         my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
         $self->api->reload($req);
-        return [ 200, [ @{ $req->{respHeaders} } ], [ $req->{respBody} ] ];
+        return [ 200, [ $req->spliceHdrs ], [ $req->{respBody} ] ];
     };
 }
 
@@ -126,40 +128,58 @@ sub _authAndTrace {
     eval "require $type";
     die $@ if ($@);
     my ( $res, $session ) = $type->run( $req, $self->{rule} );
-    $self->portal( $type->tsv->{portal}->() );
+    eval { $self->portal( $type->tsv->{portal}->() ) };
+    $self->logger->warn($@)  if $@;
     $req->userData($session) if ($session);
 
     if ( $res < 300 ) {
         if ($noCall) {
-            return [ $res, $req->{respHeaders}, [] ];
+            return [ $res, [ $req->spliceHdrs ], [] ];
         }
         else {
             $self->logger->debug('User authenticated, calling handler()');
             $res = $self->handler($req);
-            push @{ $res->[1] }, @{ $req->{respHeaders} };
+            push @{ $res->[1] }, $req->spliceHdrs;
+            return $res;
         }
-        return $res;
     }
     elsif ( $res < 400 ) {
-        return [ $res, $req->{respHeaders}, [] ];
+        if ( $req->wantJSON ) {
+            my %h    = ( $req->spliceHdrs );
+            my $host = $req->env->{HTTP_HOST};
+            if (    $h{Location}
+                and $h{Location} =~ m#^\Q$self->{portal}\E#
+                and $h{Location} !~ m#^https?://$host# )
+            {
+                return [
+                    401, [ 'WWW-Authenticate' => 'SSO ' . $self->{portal} ], []
+                ];
+            }
+        }
+        return [ $res, [ $req->spliceHdrs ], [] ];
     }
     else {
-        my %h = $req->{respHeaders} ? @{ $req->{respHeaders} } : ();
-        my $s = $type->tsv->{portal}->() . "?lmError=$res";
+        my $s = ( $self->portal ? $self->portal . "/lmerror/$res" : '' );
         $s =
             '<html><head><title>Redirection</title></head><body>'
           . qq{<script type="text/javascript">window.location='$s'</script>}
           . '<h1>Please wait</h1>'
           . qq{<p>An error occurs, you're going to be redirected to <a href="$s">$s</a>.</p>}
           . '</body></html>';
-        $h{'Content-Type'}   = 'text/html';
-        $h{'Content-Length'} = length $s;
-        return [ $res, [%h], [$s] ];
+        return [
+            $res,
+            [
+                $req->spliceHdrs,
+                'Content-Type'   => 'text/html',
+                'Content-Length' => length $s
+            ],
+            [$s]
+        ];
     }
 }
 
 ## @method hashRef user()
-# @return hash of user datas
+# @return hash of user data
 sub user {
     my ( $self, $req ) = @_;
     return $req->userData
@@ -167,6 +187,13 @@ sub user {
           || _whatToTrace => 'anonymous' };
 }
 
+## @method hashRef custom()
+# @return hash of custom data
+sub custom {
+    my ( $self, $req ) = @_;
+    return { $Lemonldap::NG::Handler::Main::tsv->{customToTrace} };
+}
+
 ## @method string userId()
 # @return user identifier to log
 sub userId {
@@ -186,14 +213,14 @@ sub group {
 }
 
 ## @method PSGI::Response sendError($req,$err,$code)
-# Add user di to $err before calling Lemonldap::NG::Common::PSGI::sendError()
+# Add user id to $err before calling Lemonldap::NG::Common::PSGI::sendError()
 # @param $req Lemonldap::NG::Common::PSGI::Request
 # @param $err String to push
 # @code int HTTP error code (default to 500)
 sub sendError {
     my ( $self, $req, $err, $code ) = @_;
     $err ||= $req->error;
-    $err = '[' . $self->userId($req) . "] $err";
+    $self->userLogger->warn( '[' . $self->userId($req) . "] $err" );
     return $self->Lemonldap::NG::Common::PSGI::sendError( $req, $err, $code );
 }
 
