@@ -21,20 +21,22 @@ BEGIN {
     }
 }
 
-has customFunctions => ( is => 'rw', isa => 'Maybe[Str]' );
-has useSafeJail     => ( is => 'rw', isa => 'Maybe[Int]' );
-has jail            => ( is => 'rw' );
-has error           => ( is => 'rw' );
+has customFunctions      => ( is => 'rw', isa => 'Maybe[Str]' );
+has useSafeJail          => ( is => 'rw', isa => 'Maybe[Int]' );
+has multiValuesSeparator => ( is => 'rw', isa => 'Maybe[Str]' );
+has jail                 => ( is => 'rw' );
+has error                => ( is => 'rw' );
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 our @builtCustomFunctions;
 
 ## @imethod protected build_jail()
 # Build and return the security jail used to compile rules and headers.
 # @return Safe object
 sub build_jail {
-    my ( $self, $api, $require ) = @_;
-
+    my ( $self, $api, $require, $dontDie ) = @_;
+    my $build = 1;
+    
     return $self->jail
       if (  $self->jail
         and $self->jail->useSafeJail
@@ -52,29 +54,34 @@ sub build_jail {
                 eval { require $f; };
             }
             if ($@) {
-                die "Unable to load '$f': $@";
+                $dontDie
+                  ? $api->logger->error($@)
+                  : die "Unable to load '$f': $@";
+                undef $build;
             }
         }
     }
 
-    @builtCustomFunctions =
-      $self->customFunctions ? split( /\s+/, $self->customFunctions ) : ();
-    foreach (@builtCustomFunctions) {
-        no warnings 'redefine';
-        $api->logger->debug("Custom function : $_");
-        my $sub = $_;
-        unless (/::/) {
-            $sub = "$self\::$_";
-        }
-        else {
-            s/^.*:://;
-        }
-        next if ( $self->can($_) );
-        eval "sub $_ {
+    if ($build) {
+        @builtCustomFunctions =
+          $self->customFunctions ? split( /\s+/, $self->customFunctions ) : ();
+        foreach (@builtCustomFunctions) {
+            no warnings 'redefine';
+            $api->logger->debug("Custom function: $_");
+            my $sub = $_;
+            unless (/::/) {
+                $sub = "$self\::$_";
+            }
+            else {
+                s/^.*:://;
+            }
+            next if ( $self->can($_) );
+            eval "sub $_ {
             return $sub(\@_)
         }";
-        $api->logger->error($@) if ($@);
-        $_ = "&$_";
+            $api->logger->error($@) if ($@);
+            $_ = "&$_";
+        }
     }
 
     if ( $self->useSafeJail ) {
@@ -88,7 +95,18 @@ sub build_jail {
     $self->jail->share_from( 'Lemonldap::NG::Common::Safelib',
         $Lemonldap::NG::Common::Safelib::functions );
 
-    $self->jail->share_from( __PACKAGE__, [ @builtCustomFunctions, '&encrypt', '&token' ] );
+    # Closure for listMatch
+    {
+        no warnings 'redefine';
+        *listMatch = sub {
+            return Lemonldap::NG::Common::Safelib::listMatch(
+                $self->multiValuesSeparator, @_ );
+        };
+    }
+
+    $self->jail->share_from( __PACKAGE__,
+        [ @builtCustomFunctions, '&encrypt', '&token', '&listMatch' ] );
+
     $self->jail->share_from( 'MIME::Base64', ['&encode_base64'] );
 
     #$self->jail->share_from( 'Lemonldap::NG::Handler::Main', ['$_v'] );
