@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.8';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -97,10 +97,10 @@ sub checkType {
 
 ## @rmethod int run
 # Check configuration and launch Lemonldap::NG::Handler::Main::run().
-# Each $checkTime, the Apache child verify if its configuration is the same
+# Each $checkTime, server child verifies if its configuration is the same
 # as the configuration stored in the local storage.
 # @param $rule optional Perl expression to grant access
-# @return Apache constant
+# @return constant
 
 sub run {
     my ( $class, $req, $rule, $protection ) = @_;
@@ -121,11 +121,10 @@ sub run {
         }
     }
 
-    # Cross domain authentication
+    # Authentication process
     my $uri = $req->{env}->{REQUEST_URI};
-
-    $uri = $req->{env}->{REQUEST_URI};
     my ($cond);
+
     ( $cond, $protection ) = $class->conditionSub($rule) if ($rule);
     $protection = $class->isUnprotected( $req, $uri ) || 0
       unless ( defined $protection );
@@ -193,6 +192,11 @@ sub run {
         $class->cleanHeaders($req);
         return $class->OK;
     }
+    elsif ( $protection == $class->MAYSKIP
+        and $class->grant( $req, $session, $uri, $cond ) eq '999_SKIP' )
+    {
+        return $class->OK;
+    }
 
     else {
 
@@ -268,11 +272,38 @@ sub checkMaintenanceMode {
 # @return True if the user is granted to access to the current URL
 sub grant {
     my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
+    my $level;
+
     return $cond->( $req, $session ) if ($cond);
 
     $vhost ||= $class->resolveAlias($req);
-    if ( my $level = $class->tsv->{authnLevel}->{$vhost} ) {
+
+    # Using URL authentification level if exists
+    for (
+        my $i = 0 ;
+        $i < ( $class->tsv->{locationCount}->{$vhost} || 0 ) ;
+        $i++
+      )
+    {
+        if ( $uri =~ $class->tsv->{locationRegexp}->{$vhost}->[$i] ) {
+            $level = $class->tsv->{locationAuthnLevel}->{$vhost}->[$i];
+            last;
+        }
+    }
+    $level
+      ? $class->logger->debug(
+        'Found AuthnLevel=' . $level . ' for "' . "$vhost$uri" . '"' )
+      : $class->logger->debug("No URL authentication level found...");
+
+    # Using VH authentification level if exists
+    if ( $level ||= $class->tsv->{authnLevel}->{$vhost} ) {
         if ( $session->{authenticationLevel} < $level ) {
+            $class->logger->debug(
+                "User authentication level = $session->{authenticationLevel}");
+            $class->logger->debug("Required authentication level = $level");
+            $class->logger->warn(
+"User rejected due to insufficient authentication level -> Session upgrade enabled"
+            );
             $session->{_upgrade} = 1;
             return 0;
         }
@@ -308,7 +339,10 @@ sub grant {
 # @return Constant $class->FORBIDDEN
 sub forbidden {
     my ( $class, $req, $session, $vhost ) = @_;
-    my $uri = $req->{env}->{REQUEST_URI};
+    my $uri    = $req->{env}->{REQUEST_URI};
+    my $portal = $class->tsv->{portal}->();
+    $portal = ( $portal =~ m#^https?://([^/]*).*# )[0];
+    $portal =~ s/:\d+$//;
     $vhost ||= $class->resolveAlias($req);
 
     if ( $session->{_logout} ) {
@@ -331,11 +365,12 @@ sub forbidden {
         $session->{ $class->tsv->{whatToTrace} } );
 
     # Redirect or Forbidden?
-    if ( $class->tsv->{useRedirectOnForbidden} ) {
-        $class->logger->debug("Use redirect for forbidden access");
+    if ( $class->tsv->{useRedirectOnForbidden} && $vhost ne $portal ) {
+        $class->logger->debug("Use redirect for forbidden access ");
         return $class->goToError( $req, $uri, 403 );
     }
     else {
+        $class->logger->debug("Self protected Portal URL") if $vhost eq $portal;
         $class->logger->debug("Return forbidden access");
         return $class->FORBIDDEN;
     }
@@ -406,15 +441,17 @@ sub goToError {
 # @return Value of the cookie if found, 0 else
 sub fetchId {
     my ( $class, $req ) = @_;
-    my $t                 = $req->{env}->{HTTP_COOKIE} or return 0;
-    my $vhost             = $class->resolveAlias($req);
+    my $t     = $req->{env}->{HTTP_COOKIE} or return 0;
+    my $vhost = $class->resolveAlias($req);
+    $class->logger->debug("VH $vhost is HTTPS")
+      if $class->_isHttps( $req, $vhost );
     my $lookForHttpCookie = ( $class->tsv->{securedCookie} =~ /^(2|3)$/
           and not $class->_isHttps( $req, $vhost ) );
     my $cn = $class->tsv->{cookieName};
     my $value =
       $lookForHttpCookie
       ? ( $t =~ /${cn}http=([^,; ]+)/o ? $1 : 0 )
-      : ( $t =~ /$cn=([^,; ]+)/o       ? $1 : 0 );
+      : ( $t =~ /$cn=([^,; ]+)/o ? $1 : 0 );
 
     if ( $value && $lookForHttpCookie && $class->tsv->{securedCookie} == 3 ) {
         $value = $class->tsv->{cipher}->decryptHex( $value, "http" );
@@ -449,6 +486,9 @@ sub retrieveSession {
         $class->logger->debug("Get session $id from Handler internal cache");
         return $class->data;
     }
+    else {
+        $class->data( {} );
+    }
 
     # 2. Get the session from cache or backend
     my $session = $req->data->{session} = (
@@ -542,11 +582,8 @@ sub retrieveSession {
 # Returns the port on which this vhost is accessed
 # @param $s VHost name
 # @return PORT
-
 sub _getPort {
-
     my ( $class, $req, $vhost ) = @_;
-
     if ( defined $class->tsv->{port}->{$vhost}
         and ( $class->tsv->{port}->{$vhost} > 0 ) )
     {
@@ -563,15 +600,14 @@ sub _getPort {
         }
     }
 }
-## @cmethod private boot _isHttps(string s)
-# Returns whether this VHost should he accessed
+
+## @cmethod private bool _isHttps(string s)
+# Returns whether this VHost should be accessed
 # via HTTPS
 # @param $s VHost name
-# @return RUE if the vhost should be accessed over HTTPS
+# @return TRUE if the vhost should be accessed over HTTPS
 sub _isHttps {
-
     my ( $class, $req, $vhost ) = @_;
-
     if ( defined $class->tsv->{https}->{$vhost}
         and ( $class->tsv->{https}->{$vhost} > -1 ) )
     {
@@ -735,15 +771,11 @@ sub abort {
 sub localUnlog {
     my ( $class, $req, $id ) = @_;
     $class->logger->debug('Local handler logout');
-    if ( $id //= $class->fetchId($req) ) {
 
-        # Delete thread data
-        if (    $class->data->{_session_id}
-            and $id eq $class->data->{_session_id} )
-        {
-            $class->data( {} );
-        }
-        delete $req->data->{session};
+    # Delete thread data
+    delete $req->data->{session};
+    $class->data( {} );
+    if ( $id //= $class->fetchId($req) ) {
 
         # Delete local cache
         if (    $class->tsv->{refLocalStorage}
